#!/usr/bin/env node
"use strict";

var _get = _interopRequireDefault(require("lodash/get"));

var _commander = _interopRequireDefault(require("commander"));

var _ = _interopRequireDefault(require(".."));

var _helpers = require("../helpers");

var _package = _interopRequireDefault(require("../../package.json"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_commander.default.version(_package.default.version).description(_package.default.description).usage('[options] <source directory>').option('-w, --watch', 'Watch files for changes').option('--root <directory>', 'Root directory for babel. This is where presets are resolved from').option('--ignored <list>', 'Ignored files and directories that match the given globs', value => value.split(',').map(item => item.trim()).filter(Boolean)).option('--ignored-for-restart <list>', 'These files are transpiled, but do not cause restart').option('--write-flow-sources', 'Write .flow files that are symlinked to source files. Helps with monorepos in some cases').option('--source-maps [true|false|inline]', 'Generate source maps for transpiled files').option('--disable-cache', 'Force retranspile all files ignoring cache').option('--keep-extra-files', 'Do NOT delete extra files in the output directory').option('-o, --output-directory <directory>', 'Output directory to write transpiled files to').option('-x, --execute <entryFile>', 'Relative path of file to execute (only supported in watcher mode)').option('--execute-delay <delay>', 'Delay in ms to in between restarts of executed file (defaults to 1000ms)', value => parseInt(value, 10) || 1000).option('--typescript', 'Enables typescript support by processing .ts and .tsx files').on('--help', () => {
  console.log('\nArguments after -- will be passed as-are to the program specified in -x flag');
  console.log('Supported NodeJS CLI flags: ', _helpers.SUPPORTED_FLAGS.join(', '));
}).allowUnknownOption().parse(process.argv);

if (_commander.default.args.length < 1) {
  _commander.default.outputHelp();

  process.exit(1);
}

if (typeof _commander.default.outputDirectory === 'undefined') {
  console.log('ERROR: You must specify output directory');

  _commander.default.outputHelp();

  process.exit(1);
}

const nodeFlags = [];

_helpers.SUPPORTED_FLAGS.forEach(item => {
  const flagIndex = _commander.default.rawArgs.indexOf(item);

  if (flagIndex !== -1) {
    let flagValue = _commander.default.rawArgs[flagIndex + 1] || true;

    if (typeof flagValue === 'undefined' || typeof flagValue === 'string' && (flagValue.startsWith('-') || _commander.default.args.includes(flagValue))) {
      // ^ If it's got no value, or value is an option or value is present in args
      // Then the value must not be own, so treat this as a boolean
      flagValue = true;
    }

    nodeFlags.push(item);

    if (flagValue !== true) {
      // ^ True values are unnecessary to specify
      nodeFlags.push(flagValue);
    }
  }
});

const config = {
  root: (0, _get.default)(_commander.default, 'root', process.cwd()),
  watch: (0, _get.default)(_commander.default, 'watch', false),
  ignored: (0, _get.default)(_commander.default, 'ignored', []),
  ignoredForRestart: (0, _get.default)(_commander.default, 'ignoredForRestart', null),
  disableCache: (0, _get.default)(_commander.default, 'disableCache', false),
  writeFlowSources: (0, _get.default)(_commander.default, 'writeFlowSources', false),
  keepExtraFiles: (0, _get.default)(_commander.default, 'keepExtraFiles', false),
  sourceMaps: (0, _get.default)(_commander.default, 'sourceMaps', false),
  sourceDirectory: _commander.default.args[0],
  outputDirectory: _commander.default.outputDirectory,
  typescript: _commander.default.typescript,
  nodeFlags,
  programFlags: _commander.default.args.slice(1),
  execute: (0, _get.default)(_commander.default, 'execute', ''),
  executeDelay: (0, _get.default)(_commander.default, 'executeDelay', 250)
};

if (config.sourceMaps !== 'inline' && typeof config.sourceMaps === 'string') {
  config.sourceMaps = config.sourceMaps === 'true';
}

if (!config.watch && config.execute) {
  console.error('ERROR: --execute is not supported without --watch');
  process.exit(1);
}

(0, _.default)(config).catch(error => {
  (0, _helpers.logError)(error);
  process.exit(1);
});